\name{rstan-internal}
\alias{rstan-internal}
\alias{get_cppcode,ANY-method}
\alias{get_logposterior,ANY-method}
\alias{get_adaptation_info,ANY-method}
\alias{get_sampler_params,ANY-method}
\alias{get_posterior_mean,ANY-method}
\alias{get_elapsed_time,ANY-method}
\alias{get_num_upars,ANY-method}
\alias{log_prob,ANY-method}
\alias{grad_log_prob,ANY-method}
\alias{unconstrain_pars,ANY-method}
\alias{constrain_pars,ANY-method}
\alias{cpp_object_initializer}
\alias{cxxdso-class}
\alias{show,cxxdso-method}
\alias{grab_cxxfun,cxxdso-method}
\alias{is_dso_loaded,cxxdso-method}
\alias{RNG}
\alias{OUT}
\title{Internal Functions and Methods}
\description{
These functions are not intended to be called directly. In the case of
methods, they are documented elsewhere, either with the associated generic
function or with the class of the object for which the method is defined.
}
\keyword{internal}


