\name{rstan.package.skeleton}
\alias{rstan.package.skeleton}
\title{Create a Skeleton for a New Source Package with Stan Programs}
\description{
  This function has been removed from \pkg{rstan}. Please use the new 
  \code{rstan_package_skeleton} function in the \pkg{rstantools} package.
}
